# random triangle on 0..1 x 0..1
#

from time import time
from random import random
from math import *
from statistics import *

# Monte-Carlo
#
def RandomTriangle_MC(N):
    areaSum = 0
    for k in range(N):
        X = [ random() for i in range(3) ]
        Y = [ random() for i in range(3) ]
        area = abs(0.5*( (X[1]-X[2])*(Y[1]+Y[2]) +(X[2]-X[0])*(Y[2]+Y[0]) +(X[0]-X[1])*(Y[0]+Y[1]) ))
        areaSum+=area
    return areaSum/N

# regular grid
#
def RandomTriangle_RG(M):
    n = M
    dx = 1.0/n
    Rs = 0+dx/2
    
    areaSum = 0
    for i1 in range(n):
        x1 = Rs+i1*dx
        for i2 in range(n):
            x2 = Rs+i2*dx
            for i3 in range(n):
                x3 = Rs+i3*dx
                for i4 in range(n):
                    y1 = Rs+i4*dx
                    for i5 in range(n):
                        y2 = Rs+i5*dx
                        for i6 in range(n):
                            y3 = Rs+i6*dx

                            area = abs(0.5*( (x1-x2)*(y1+y2) +(x2-x3)*(y2+y3) +(x3-x1)*(y3+y1) ))
                            areaSum += area
    return areaSum/n**6

start = time()

M = 8
avg = RandomTriangle_RG(M)
print("regular area: {}, N={}".format(avg, M**6))
print("...rev=",1/avg)

N = M**6
res = []
for k in range(10):
    avg = RandomTriangle_MC(N)
    res.append(avg)

avg  = fmean(res)
dev = pstdev(res, avg)
print("average area: {}±{}, N={}".format(avg, dev, N))
print("...rev=",1/avg)

finish = time()
print("elapsed=", finish-start)

"""
M=10
regular area: 0.07554299999988685, N=1000000)
...rev= 13.237493877678908
average area: 0.07640171086030129±4.336911783891917e-05, N=1000000)
...rev= 13.088712134057786
elapsed= 11.944649457931519

M=12
regular area: 0.07581839018541255, N=2985984
...rev= 13.189412193460155
average area: 0.0763895069038637±2.8734916665961632e-05, N=2985984
...rev= 13.090803181364967
elapsed= 35.4831817150116
"""
